using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace BmxCs {
	public interface INumberGenerator {
		int Next(int maxNumber);
	}

	/// <summary>
	/// System.RandomgpAINumberGenerator̎B
	/// </summary>
	public class RandomGenerator : INumberGenerator {
		Random rnd;

		public RandomGenerator(int seed) {
			rnd = new Random(seed);
		}
		public RandomGenerator() {
			rnd = new Random();
		}

		#region INumberGenerator o

		public int Next(int maxNumber) {
			return rnd.Next(maxNumber) + 1;
		}

		#endregion
	}

	/// <summary>
	/// Œe[u𗘗pAINumberGenerator̎
	/// </summary>
	public class TableGenerator : INumberGenerator {
		int[] table;
		int index;

		public TableGenerator(int[] table) {
			this.table = table;
			index = 0;
		}

		#region INumberGenerator o

		public int Next(int maxNumber) {
			if (index == table.Length) index = 0;
			return 1 + table[index++] % maxNumber;
		}

		#endregion
	}

	/// <summary>
	/// BMxt@Cǂݍ݁ABmxHeaderBmxScoreԂNXB
	/// </summary>
	public static class BmxLoader {

		private class BranchHelper {
			public enum BranchMode {
				RANDOM, SWITCH
			}

			private BranchMode _mode;

			public BranchMode Mode {
				get { return _mode; }
				set { _mode = value; }
			}
			private int _generated;

			public int Generated {
				get { return _generated; }
				set { _generated = value; }
			}
			private int? _now;

			public int? Now {
				get { return _now; }
				set { _now = value; }
			}
			private bool _skip;

			public bool Skip {
				get { return _skip; }
				set { _skip = value; }
			}

			public BranchHelper(BranchMode mode, int pattern) {
				_mode = mode;
				_generated = pattern;
				_now = null;
				_skip = false;
			}

			public bool Status {
				get {
					return (Mode == BranchMode.SWITCH && !Skip)
						|| Now == null
						|| Generated == Now
						|| (Now < 0 && -Generated != Now);
					//if (_mode == BranchMode.SWITCH && !_skip)
					//    return true;
					//if (_now == null)
					//    return true;
					//if (_generated == _now)
					//    return true;
					//if (_now < 0 && -_generated != _now)
					//    return true;

					//return false;
				}
			}
		}

		public static Bmx Load(string filename) {
			return Load(filename, null);
		}

		public static Bmx Load(string filename, INumberGenerator pattern) {
			StreamReader reader = new StreamReader(new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read), Encoding.GetEncoding("shift-jis"));

			Bmx ret = new Bmx(filename);

			Stack<BranchHelper> random = new Stack<BranchHelper>();

			if (pattern == null)
				pattern = new RandomGenerator();

			while (!reader.EndOfStream) {
				string line = reader.ReadLine();

				// ƂāAȉ̓̓wb_ƂĎ擾B
				if (line.StartsWith("%url", StringComparison.CurrentCultureIgnoreCase)
					|| line.StartsWith("%mail", StringComparison.CurrentCultureIgnoreCase))

					line = "#" + line.Substring(1);
				

				if (line.Length > 3 && line[0] == '#') {
					// LȕH
					
					int i = 0;

					do 
						i++;
					while (i < line.Length && !char.IsWhiteSpace(line[i]) && line[i] != ':');

					if (i == line.Length) i--;

					char separator = line[i];


					string head = line.Substring(1, i).Trim().ToUpper();
					string description = line.Substring(i + 1).Trim();

					int pat;
					switch (head) {
						case "RANDOM":
							random.Push(new BranchHelper(BranchHelper.BranchMode.RANDOM, pat = pattern.Next(int.Parse(description))));
							ret.Pattern.Add(pat);
							continue;
						case "SETRANDOM":
							random.Push(new BranchHelper(BranchHelper.BranchMode.RANDOM, int.Parse(description)));
							continue;
						case "IF":
							if (random.Peek().Mode != BranchHelper.BranchMode.RANDOM) continue;
							random.Peek().Now = int.Parse(description);
							continue;
						case "ELSE":
							if (random.Peek().Mode != BranchHelper.BranchMode.RANDOM) continue;
							random.Peek().Now = -random.Peek().Now;
							continue;
						case "ENDIF":
							if (random.Peek().Mode != BranchHelper.BranchMode.RANDOM) continue;
							random.Peek().Now = null;
							continue;
						case "ENDRANDOM":
							if (random.Peek().Mode != BranchHelper.BranchMode.RANDOM) continue;
							random.Pop();
							continue;

						case "SWITCH":
							if (random.Peek().Skip) continue;
							random.Push(new BranchHelper(BranchHelper.BranchMode.SWITCH, pat = pattern.Next(int.Parse(description))));
							ret.Pattern.Add(pat);
							continue;
						case "SETSWITCH":
							if (random.Peek().Skip) continue;
							random.Push(new BranchHelper(BranchHelper.BranchMode.SWITCH, int.Parse(description)));
							continue;
						case "CASE":
							if (random.Peek().Mode != BranchHelper.BranchMode.SWITCH) continue;
							if (random.Peek().Skip) continue;
							if (!random.Peek().Now.HasValue) 
								random.Peek().Now = int.Parse(description);
							continue;
						case "SKIP":
							if (random.Peek().Mode != BranchHelper.BranchMode.SWITCH) continue;
							random.Peek().Skip = true;
							continue;
						case "DEF":
							if (random.Peek().Mode != BranchHelper.BranchMode.SWITCH) continue;
							if (random.Peek().Skip) continue;
							if (!random.Peek().Now.HasValue)
								random.Peek().Now = random.Peek().Generated;
							continue;
						case "ENDSW":
							if (random.Peek().Mode != BranchHelper.BranchMode.SWITCH) continue;
							random.Pop();
							continue;

						default:
							if (random.Count != 0 && !random.Peek().Status)
								continue;
							break;
					}

					// LȃZp[^ȂƂɂ͂ꎩ̂wb_
					if (i == line.Length) {
						ret.Header[head] = "";
					}
					// Xy[Xŋ؂Ăꍇ͒lt̃wb_
					else if (char.IsWhiteSpace(separator)) {
						ret.Header[head] = description;
					}
					// Z~Rŋ؂Ăꍇ̓^C~Oe[u
					else if (separator == ':') {
						string table = line.Substring(i + 1);

						int barnum = int.Parse(line.Substring(1, 3));
						int ch = int.Parse(line.Substring(4, 2), System.Globalization.NumberStyles.AllowHexSpecifier);

						List<float> data;

						if (ch == 2) {
							// 2ԃ`l(ߒύX)́AƂēǂݍ
							data = new List<float>(1);

							data.Add(float.Parse(table));
						}
						else if (ch == 3) {
							// 3ԃ`l(BPMύX)16iƂēǂݍ
							data = new List<float>(4);

							for (int j = 0; j < table.Length; j += 2)
								data.Add((float)int.Parse(table.Substring(j, 2).Trim(), System.Globalization.NumberStyles.AllowHexSpecifier));
						}
						else {
							data = new List<float>(4);

							for (int j = 0; j < table.Length; j += 2)
								data.Add((float)Base36.ToInt(table.Substring(j).Trim()));

						}

						ret.Score[ch].GetBarEvent(barnum).AddData(data);

					}
					


					

				}
			}


			return ret;
		}
	}
}
