using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using BmxCs;

namespace bmxCsTest {
	public partial class Form3 : Form {
		BmxTimingTable table;

		double draw_scale = 0.0;

		private void SetDrawScale(double d) {
			draw_scale = d;
			pictureBox1.ClientSize = new Size(512, (int)(table.TimeLength * draw_scale + 100));

			using (Graphics g = pictureBox1.CreateGraphics()) {
				PutAll(g);
			}
		}

		private void PutAll(Graphics g) {
			PutScore(g);
			PutNotes(g);
		}

		public Form3(BmxTimingTable _table) {
			InitializeComponent();

			this.table = _table;
		}

		private void Form3_Load(object sender, EventArgs e) {
			SetDrawScale(100.0);
		}

		private void PutNotes(Graphics g) {

			foreach (Event<NoteEvent> ev in table.Notes) {
				Brush b;
				Pen p;
				switch (ev.Data.Type) {
					case NoteEvent.NoteType.Normal:
						b = Brushes.White;
						p = Pens.White;
						break;
					case NoteEvent.NoteType.Hidden:
						b = Brushes.Yellow;
						p = Pens.Yellow;
						break;
					case NoteEvent.NoteType.LongStart:
						b = Brushes.Blue;
						p = Pens.Blue;
						break;
					case NoteEvent.NoteType.LongEnd:
						b = Brushes.Green;
						p = Pens.Green;
						break;
					case NoteEvent.NoteType.Mine:
						b = Brushes.Red;
						p = Pens.Red;
						break;
					default:
						b = Brushes.Gray;
						p = Pens.Gray;
						break;
				}

				int pos = ev.Data.Lane - NoteEvent.LaneType.P1_1;


				float ypos = calcDrawPosition(ev.Time);
				g.FillRectangle(b, pos * 16, ypos, 16, 4);
				g.DrawLine(p, 0, ypos, 512, ypos);
			}
		}

		private float calcDrawPosition(double t) {
			return (float)(t * draw_scale);
		}

		private void PutScore(Graphics g) {
			Brush[] b = { Brushes.DarkGray, Brushes.DarkTurquoise };
			int i = 0;
			int barid = 0;
			foreach (Event<ScoreEvent> ev in table.Score) {
				if (ev.Data.Type == ScoreEvent.EventType.BarLine) {
					float ypos = calcDrawPosition(ev.Time);
					float height = calcDrawPosition(ev.Data.Val);
					g.FillRectangle(b[i], 0, ypos, 512, height);
					i = 1 - i;
					g.DrawString((barid++).ToString(), this.Font, b[i], 0, ypos);
				}
				else if (ev.Data.Type == ScoreEvent.EventType.Stop) {
					float ypos = calcDrawPosition(ev.Time);
					float height = calcDrawPosition(ev.Data.Val);
					g.FillRectangle(Brushes.Navy, 0, ypos, 512, height);
					i = 1 - i;
				}
			}
		}

		private void PutMeasure() { }

		private void pictureBox1_Paint(object sender, PaintEventArgs e) {
			PutAll(e.Graphics);
		}
	}
}